#include "BehaviorBase.h"
#include "TreePopulation.h"
#include "SimManager.h"
#include <stdio.h>
#include <math.h>
/*////////////////////////////////////////////////////////////////////////////
Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
clBehaviorBase::clBehaviorBase(clSimManager *p_oSimManager)
   : clWorkerBase(p_oSimManager) {
  try {
    m_fVersionNumber = 0;
    m_fMinimumVersionNumber = 0;
    m_iNumSpeciesTypeCombos = 0;
    m_iNumBehaviorSpecies = 0;
    mp_whatSpeciesTypeCombos = NULL;
    mp_iWhatSpecies = NULL;
    m_iNewTreeInts = 0;
    m_iNewTreeFloats = 0;
    m_iNewTreeChars = 0;
    m_iNewTreeBools = 0;
  }
  catch(modelErr &err) {throw(err);}
  catch (modelMsg &msg) {throw(msg);} //non-fatal error
  catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clBehaviorBase::clBehaviorBase");
   throw(stcErr);
 }
}
//----------------------------------------------------------------------------

/*////////////////////////////////////////////////////////////////////////////
Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
clBehaviorBase::~clBehaviorBase() {
    delete[] mp_whatSpeciesTypeCombos;
    delete[] mp_iWhatSpecies;
}
//----------------------------------------------------------------------------

/*////////////////////////////////////////////////////////////////////////////
SetSpeciesTypeCombos
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
void clBehaviorBase::SetSpeciesTypeCombos(
      short int iNumCombos, stcSpeciesTypeCombo *p_whatCombos) {
  try {
    clPopulationBase *p_oTemp;  //for getting the tree population object
    clTreePopulation *p_oTrees; //tree population - for getting species
    short int iNumSpecies, iNumTypes, //for validating species and type numbers
              *p_iSpeciesList,  //for compiling a list of unique species
              i, j;             //loop counters
    bool bFound;                //for checking species for uniqueness

    if (mp_whatSpeciesTypeCombos) {
      //we've already got it - can't do this again!
      modelErr stcErr;
      strcpy(stcErr.cFunction, "clBehaviorBase::SetSpeciesTypeCombos");
      stcErr.iErrorCode = DATA_READ_ONLY;
      strcpy(stcErr.cMoreInfo, "behavior species/type data");
      throw(stcErr);
    }
    m_iNumSpeciesTypeCombos = iNumCombos;

    //ERROR CHECKING
    //Check to see if we have any combos
    if (0 >= m_iNumSpeciesTypeCombos) {
      //We have no combos  - if the pointer is NULL, exit without error
      if (NULL == p_whatCombos) return;
      //Otherwise - problem - throw error
      else {
        modelErr stcErr;
        strcpy(stcErr.cFunction, "clBehaviorBase::SetSpecies");
        stcErr.iErrorCode = BAD_DATA;
        strcpy(stcErr.cMoreInfo,
          "Number of combos doesn't match species/type combo list.");
        throw(stcErr);
      }
    //Number of combos in the list isn't zero - make sure the pointer isn't NULL
    } else {
      if (NULL == p_whatCombos) { //throw error
        modelErr stcErr;
        strcpy(stcErr.cFunction, "clBehaviorBase::SetSpecies");
        stcErr.iErrorCode = BAD_DATA;
        strcpy(stcErr.cMoreInfo,
          "Number of combos doesn't match species/type combo list.");
        throw(stcErr);
      }
    }

    //Create a new array and copy in the values
    mp_whatSpeciesTypeCombos = new stcSpeciesTypeCombo[m_iNumSpeciesTypeCombos];
    for (i = 0; i < m_iNumSpeciesTypeCombos; i++) {
      mp_whatSpeciesTypeCombos[i].iSpecies = p_whatCombos->iSpecies;
      mp_whatSpeciesTypeCombos[i].iType = p_whatCombos->iType;
      p_whatCombos++;
    }

    //Validate that all the species and types are good
    p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
    p_oTrees = (clTreePopulation*)p_oTemp;
    iNumSpecies = p_oTrees->GetNumberOfSpecies();
    iNumTypes = p_oTrees->GetNumberOfTypes();
    for (i = 0; i < m_iNumSpeciesTypeCombos; i++) {
      if (mp_whatSpeciesTypeCombos[i].iSpecies < 0 ||
          mp_whatSpeciesTypeCombos[i].iSpecies >= iNumSpecies) {
        //Bad species - throw error
        modelErr stcErr;
        strcpy(stcErr.cFunction, "clBehaviorBase::SetSpecies");
        stcErr.iErrorCode = BAD_DATA;
        sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species number:  ",
          mp_whatSpeciesTypeCombos[i].iSpecies);
        throw(stcErr);
      }
      if (mp_whatSpeciesTypeCombos[i].iType < 0 ||
          mp_whatSpeciesTypeCombos[i].iType >= iNumTypes) {
        //Bad types - throw error
        modelErr stcErr;
        strcpy(stcErr.cFunction, "clBehaviorBase::SetTypes");
        stcErr.iErrorCode = BAD_DATA;
        sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized type:  ",
           mp_whatSpeciesTypeCombos[i].iType);
        throw(stcErr);
      }
    }

    //Assemble the list of unique species present and put that in the species
    //array

    //Declare the temp. species array to be as big as the combo list to make
    //sure we have space for everything, and initialize values to -1
    p_iSpeciesList = new short int[m_iNumSpeciesTypeCombos];
    for (i = 0; i < m_iNumSpeciesTypeCombos; i++)
      p_iSpeciesList[i] = -1;

    //Go through each combo, and for the species for that combo, if it's not
    //already on the temp list, add it
    //Re-use iNumSpecies to keep track of how many species were found
    iNumSpecies = 0;
    for (i = 0; i < m_iNumSpeciesTypeCombos; i++) {
      bFound = false;
      //Test to see if this species is already on the list
      for (j = 0; j < iNumSpecies; j++) {
        if (mp_whatSpeciesTypeCombos[i].iSpecies == p_iSpeciesList[j])
          {bFound = true; break;}
      }
      if (!bFound) {
        //Add the species to the list and increment the number of found species
        //by one
        p_iSpeciesList[iNumSpecies] = mp_whatSpeciesTypeCombos[i].iSpecies;
        iNumSpecies++;
      }
    } //end of for (i = 0; i < m_iNumSpeciesTypeCombos; i++)

    //At this point p_iSpeciesList should have the unique species list - declare
    //mp_iWhatSpecies and assign values
    m_iNumBehaviorSpecies = iNumSpecies;
    mp_iWhatSpecies = new short int[m_iNumBehaviorSpecies];
    for (i = 0; i < iNumSpecies; i++)
      mp_iWhatSpecies[i] = p_iSpeciesList[i];

    delete[] p_iSpeciesList;

  } //end of try block
  catch (modelErr &err) {throw(err);}
  catch (modelMsg &msg) {throw(msg);} //non-fatal error
  catch (...) {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clBehaviorBase::SetSpeciesTypeCombos");
    throw(stcErr);
  }
}
//----------------------------------------------------------------------------

/*/////////////////////////////////////////////////////////////////////////////
GetSpeciesTypeCombo()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////*/
struct stcSpeciesTypeCombo clBehaviorBase::GetSpeciesTypeCombo(short int iIndex) {
  try {
    //If the index isn't valid - throw error
    if (iIndex < 0 || iIndex >= m_iNumSpeciesTypeCombos) {
      modelErr stcErr;
      strcpy(stcErr.cFunction, "clBehaviorBase::GetSpeciesTypeCombo");
      stcErr.iErrorCode = BAD_DATA;
      sprintf(stcErr.cMoreInfo, "%s%d%s", "Index \"", iIndex,
        "\" is not a valid species/type combo index.");
      throw(stcErr);
    }
    else return (mp_whatSpeciesTypeCombos[iIndex]);
  }
  catch (modelErr &err) {throw(err);}
  catch (modelMsg &msg) {throw(msg);} //non-fatal error
  catch (...) {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clBehaviorBase::GetSpeciesTypeCombo");
    throw(stcErr);
  }
}
//----------------------------------------------------------------------------

/*////////////////////////////////////////////////////////////////////////////
GetBehaviorSpecies
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
short int clBehaviorBase::GetBehaviorSpecies(short int iIndex) {
 try {
   //If the index isn't valid - throw error
   if (iIndex < 0 || iIndex >= m_iNumBehaviorSpecies) {
     modelErr stcErr;
     strcpy(stcErr.cFunction, "clBehaviorBase::GetBehaviorSpecies");
     stcErr.iErrorCode = BAD_DATA;
     sprintf(stcErr.cMoreInfo, "%s%d%s", "Index \"", iIndex,
       "\" is not a valid species index.");
     throw(stcErr);
   }
   else return (mp_iWhatSpecies[iIndex]);
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clBehaviorBase::GetBehaviorSpecies");
   throw(stcErr);
 }
}
//----------------------------------------------------------------------------

/*////////////////////////////////////////////////////////////////////////////
ValidateVersionNumber
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
short int clBehaviorBase::ValidateVersionNumber(float fTestVersion) {
  //Make all comparisons to the first two digits after the decimal
  //Does this match the current version?
  if (fabs(fTestVersion - m_fVersionNumber) < 0.01)
    return 1;
  //Is it in the range?
  else if (fTestVersion >= m_fMinimumVersionNumber
           && fTestVersion <= m_fVersionNumber)
    return 0;
  else return -1;
}
//-----------------------------------------------------------------------------

/*////////////////////////////////////////////////////////////////////////////
 Virtual functions - we must have their definitions here.  If we don't we get
 a linker error
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////*/
void clBehaviorBase::Action() {;}
void clBehaviorBase::RegisterTreeDataMembers() {;}

